from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from robot.api import logger
from .util import Util

class ClickHelper(Util):

    def __init__(self):
        pass
    
    @keyword("Click On WebElement")
    def click_on_webelement(self, locator):
        """
        Wait for ``locator`` present in page then scroll to locator and perform click operation
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
            self.sellib.click_element(locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)
    
    @keyword("Click On WebElement With Retry")
    def click_on_webelement_with_retry(self, locator, retry="3x", retry_interval="2s"):
        """
        Similar to ``Click On WebElement`` with retry
        """
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumHelperLibrary.Click On WebElement", locator)


from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from robot.api import logger
from .util import Util

class LinkHelper(Util):

    def __init__(self):
        pass
    
    @keyword("Link By Text Should Be Present")
    def link_by_text_should_be_present(self, text):
        """
        Wait for link with ``text`` present in web page
         - uses ``//a[contains(normalize-space(text()),'{text}')]`` locator internally
         - ``text`` will be replaced in locator
        """
        locator = "//a[contains(normalize-space(text()),'{text}')]".format(text=text)
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)
    
    @keyword("Link By Text Should Not Be Present")
    def link_by_text_should_not_be_present(self, text):
        """
        Wait for link with ``text`` not present in web page
         - uses ``//a[contains(normalize-space(text()),'{text}')]`` locator internally
         - ``text`` will be replaced in locator
        """
        locator = "//a[contains(normalize-space(text()),'{text}')]".format(text=text)
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element_not_present(self, self.sellib, locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)

    @keyword("Click On Link By Text")
    def click_on_link_by_text(self, text):
        """
        Wait for link with ``text`` present in web page then scroll to element & perform click operation
         - uses ``//a[contains(normalize-space(text()),'{text}')]`` locator internally
         - ``text`` will be replaced in locator
        """
        locator = "//a[contains(normalize-space(text()),'{text}')]".format(text=text)
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
            self.sellib.click_element(locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)
    
    @keyword("Click On Link By Text With Retry")
    def click_on_link_by_text_with_retry(self, text, retry="3x", retry_interval="2s"):
        """
        Similar to `Click On Link By Text` with retry
        """
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumHelperLibrary.Click On Link By Text", text)
    
    @keyword("Link By Title Should Be Present")
    def link_by_title_should_be_present(self, title):
        """
        Wait for link with ``@title`` attribute value present in web page
         - uses ``//button[contains(normalize-space(@title),'{title}')]`` locator internally
         - ``title`` will be replaced in locator
        """
        locator = "//a[contains(normalize-space(@title),'{title}')]".format(title=title)
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)
    
    @keyword("Link By Title Should Not Be Present")
    def link_by_title_should_not_be_present(self, title):
        """
        Wait for link with ``@title`` attribute value not present in web page
         - uses ``//button[contains(normalize-space(@title),'{title}')]`` locator internally
         - ``title`` will be replaced in locator
        """
        locator = "//a[contains(normalize-space(@title),'{title}')]".format(title=title)
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element_not_present(self, self.sellib, locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)

    @keyword("Click On Link By Title")
    def click_on_link_by_title(self, title):
        """
        Wait for link with ``@title`` attribute value present in web page then scroll to element & perform click
         - uses ``//button[contains(normalize-space(@title),'{title}')]`` locator internally
         - ``title`` will be replaced in locator
        """
        locator = "//a[contains(normalize-space(@title),'{title}')]".format(title=title)
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
            self.sellib.click_element(locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)
    
    @keyword("Click On Link By Title With Retry")
    def click_on_link_by_title_with_retry(self, title, retry="3x", retry_interval="2s"):
        """
        Similar to `Click On Link By Title` with retry
        """
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumHelperLibrary.Click On Link By Title", title)


from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from robot.api import logger
from .util import Util

class SelectHelper(Util):

    def __init__(self):
        pass
    
    @keyword("Select Label From Picklist")
    def select_label_from_picklist(self, locator, label):
        """
        Wait for picklist with ``locator`` present in webpage then scroll to element & select option based on display text (label)
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
            self.sellib.select_from_list_by_label(locator, label)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)
    
    @keyword("Select Label From Picklist With Retry")
    def select_label_from_picklist_with_retry(self, locator, label, retry="3x", retry_interval="2s"):
        """
        Similar to `Select Label From Picklist` with retry
        """
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumHelperLibrary.Select Label From Picklist", locator, label)


from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from robot.api import logger
from .util import Util

class InputHelper(Util):

    def __init__(self):
        pass
    
    @keyword("Input Text Into Textbox")
    def input_text_into_textbox(self, locator, text):
        """
        Wait for ``locator`` present in webpage then scroll to element and input text
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
            self.sellib.input_text(locator, text)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)
    
    @keyword("Input Text Into Textbox With Retry")
    def input_text_into_textbox_with_retry(self, locator, text, retry="3x", retry_interval="2s"):
        """
        Similar to ``Input Text Into Textbox`` with retry
        """
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumHelperLibrary.Input Text Into Textbox", locator, text)
    
    @keyword("Textbox Should Contain Value")
    def textbox_should_contain_value(self, locator, text):
        """
        Wait for ``locator`` then scroll to element & validates textbox `@value` attribute contains give `text`
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
            self.sellib.textfield_should_contain(locator, text)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)


from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from robot.api import logger
from .util import Util

class TitleHelper(Util):

    def __init__(self):
        pass

    @keyword("Title Should Contain")
    def title_should_contain(self, value):
        """
        Fetch current window title and compare title contains `value`
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            title = self.sellib.get_title()
            BuiltIn().should_contain(title, value)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)


from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from robot.api import logger
from .util import Util

class FrameHelper(Util):

    def __init__(self):
        pass
    
    @keyword("Switch To Iframe")
    def switch_to_iframe(self, locator):
        """
        Wait for `iframe` with ``locator`` and switches to iframe
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
            self.sellib.select_frame(locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)



from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from robot.api import logger
from .util import Util

class AttributeHelper(Util):

    def __init__(self):
        pass
    
    @keyword("Get WebElement Attribute")
    def get_webelement_attribute(self, locator, attribute):
        """
        Wait for `locator` in webpage then fetch & return `attribute` value
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        value = None
        try:
            Util.wait_for_element(self, self.sellib, locator)
            value = self.sellib.get_element_attribute(locator, attribute)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)
        return value
    
    @keyword("WebElement Attribute Should Contain")
    def webelement_attribute_should_contain(self, locator, attribute, value):
        """
        Wait for `locator` in webpage then fetch & compare `attribute` value with `value` argument
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
            actual = self.sellib.get_element_attribute(locator, attribute)
            BuiltIn().should_contain(actual, value)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)


from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from robot.api import logger
from .util import Util

class WaitHelper(Util):

    def __init__(self):
        pass
    
    @keyword("Wait Until Element Is Visible With Retry")
    def wait_until_element_is_visible_with_retry(self, locator, retry="3x", retry_interval="2s"):
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumLibrary.Wait Until Element Is Visible", locator)
    
    @keyword("Wait Until Element Is Not Visible With Retry")
    def wait_until_element_is_not_visible_with_retry(self, locator, retry="3x", retry_interval="2s"):
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumLibrary.Wait Until Element Is Not Visible", locator)
    
    @keyword("Wait Until Page Contains With Retry")
    def wait_until_page_contains_with_retry(self, text, retry="3x", retry_interval="2s"):
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumLibrary.Wait Until Page Contains", text)
    
    @keyword("Wait Until Page Does Not Contain With Retry")
    def wait_until_page_does_not_contain_with_retry(self, text, retry="3x", retry_interval="2s"):
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumLibrary.Wait Until Page Does Not Contain", text)
    
    @keyword("Wait Until Page Contains Element With Retry")
    def wait_until_page_contains_element_with_retry(self, locator, retry="3x", retry_interval="2s"):
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumLibrary.Wait Until Page Contains Element", locator)
    
    @keyword("Wait Until Page Does Not Contain Element With Retry")
    def wait_until_page_does_not_contain_element_with_retry(self, locator, retry="3x", retry_interval="2s"):
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumLibrary.Wait Until Page Does Not Contain Element", locator)
    
    @keyword("Wait For WebElement")
    def wait_for_webelement(self, locator):
        """
        Wait for `locator` present in webpage then scroll to element
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)

    @keyword("Wait For WebElement Disappear")
    def wait_for_webelement_disappear(self, locator):
        """
        Wait for `locator` not present in webpage
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element_not_present(self, self.sellib, locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e) 

    @keyword("Wait For WebElement With Retry")
    def wait_for_webelement_with_retry(self, locator, retry="3x", retry_interval="2s"):
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumHelperLibrary.Wait For WebElement", locator)
    
    @keyword("Wait For WebElement Disappear With Retry")
    def wait_for_webelement_disappear_with_retry(self, locator, retry="3x", retry_interval="2s"):
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumHelperLibrary.Wait For WebElement Disappear", locator)

    @keyword("Wait Until DOM Loaded")
    def wait_until_dom_loaded(self):
        """
        Wait until dom loaded in web page
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            self.sellib.wait_for_condition('return window.document.readyState === "complete"')
        except Exception as e:
            Util.log_failure(self, self.sellib, e)
    
    @keyword("Wait Until Element Contains With Retry")
    def wait_until_element_contains_with_retry(self, locator, value, retry="3x", retry_interval="2s"):
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        Util.wait_for_element(self, self.sellib, locator)
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumLibrary.Wait Until Element Contains", locator, value)
    
    @keyword("Wait Until Element Does Not Contain With Retry")
    def wait_until_element_does_not_contain_with_retry(self, locator, value, retry="3x", retry_interval="2s"):
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        Util.wait_for_element(self, self.sellib, locator)
        BuiltIn().wait_until_keyword_succeeds(retry, retry_interval, "SeleniumLibrary.Wait Until Element Does Not Contain", locator, value)


from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from robot.api import logger
from .util import Util

class TextHelper(Util):

    def __init__(self):
        pass
    
    @keyword("Get WebElement Text")
    def get_webelement_text(self, locator):
        """
        Wait for `locator`then fetch and return webelement text
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        value = None
        try:
            Util.wait_for_element(self, self.sellib, locator)
            value = self.sellib.get_text(locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)
        return value
    
    @keyword("Get WebElement Value")
    def get_webelement_value(self, locator):
        """
        Wait for `locator` then fetch and return webelement `@value` attribute text
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        value = None
        try:
            Util.wait_for_element(self, self.sellib, locator)
            value = self.sellib.get_value(locator)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)
        return value


from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword, not_keyword
from robot.api import logger

class Util():

    def __init__(self):
        pass

    @not_keyword
    def wait_for_element(self, instance, locator):
        instance.wait_until_page_contains_element(locator)
        BuiltIn().run_keyword_and_ignore_error('SeleniumLibrary.Scroll Element Into View', locator)
    
    @not_keyword
    def wait_for_element_not_present(self, instance, locator):
        instance.wait_until_page_does_not_contain_element(locator)

    @not_keyword
    def log_failure(self, instance, exception):
        instance.capture_page_screenshot()
        BuiltIn().fail(exception)


import sys
from os.path import abspath, dirname, join

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


PY3 = sys.version_info > (3,)

VERSION = None
version_file = join(dirname(abspath(__file__)), 'src', 'SeleniumHelperLibrary', 'version.py')
with open(version_file) as file:
    code = compile(file.read(), version_file, 'exec')
    exec(code)

DESCRIPTION = """
Helper keywords for robotframework-selenium
"""[1:-1] 

CLASSIFIERS = """
Operating System :: OS Independent
Programming Language :: Python
Topic :: Software Development :: Testing
"""[1:-1]


setup(
      name='robotframework-seleniumhelperlibrary',
      version=VERSION,
      description='Helper keywords for robotframework-selenium',
      long_description=DESCRIPTION,
      author='Shiva Adirala',
      author_email='adiralashiva8@gmail.com',
      url='https://github.com/adiralashiva8/robotframework-seleniumhelperlibrary',
      license='MIT',
      keywords='Helper keywords for robotframework-selenium',
      platforms='any',
      classifiers=CLASSIFIERS.splitlines(),
      package_dir={'': 'src'},
      packages=['SeleniumHelperLibrary'],
      install_requires=[
          'robotframework',
          'robotframework-seleniumlibrary'
      ],)


from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from robot.api import logger
from .attribute import AttributeHelper
from .click import ClickHelper
from .frame import FrameHelper
from .input import InputHelper
from .select import SelectHelper
from .textarea import TextareaHelper
from .title import TitleHelper
from .wait import WaitHelper
from .text import TextHelper
from .link import LinkHelper
from .button import ButtonHelper
from .webelement import WebElementHelper
from .util import Util

class SeleniumHelperLibrary(AttributeHelper, ClickHelper,
 FrameHelper, InputHelper, SelectHelper, TextareaHelper, TitleHelper, WaitHelper,
 LinkHelper, ButtonHelper, WebElementHelper, TextHelper):

    """
    Core Principal:
      Achieve synchronization before performing any action on ``WebElement``

    Every ``Keyword`` consist following steps
        - Wait For WebElement
        - Scroll To WebElement (ignores scroll issue)
        - Perform Respective Action
    
    """

    ROBOT_LIBRARY_SCOPE = 'GLOBAL'
    ROBOT_LIBRARY_VERSION = VERSION

    def __init__(self):
        pass

    @keyword("Scroll By Coordinates")
    def scroll_by_coordinates(self, x, y):
        """
        Perform scroll in webpage based on co-ordinates
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        js = "window.scroll({x},{y})".format(x=x, y=y)
        self.sellib.execute_javascript(js)

    @keyword("Should Contain With Screenshot")
    def should_contain_with_screenshot(self, actual, expected):
        """
        Validates `actual` contains `expected` value.
         - Captures screenshot if failed
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        isPassed = BuiltIn().run_keyword_and_return_status("BuiltIn.Should Contain", actual, expected)
        if not isPassed:
            error_msg = "{expected_value} is not found in {actual_value}".format(expected_value=expected, actual_value=actual)
            Util.log_failure(self, self.sellib, error_msg)
    
    @keyword("Should Not Contain With Screenshot")
    def should_not_contain_with_screenshot(self, actual, expected):
        """
        Validates `actual` not contains `expected` value.
         - Captures screenshot if failed
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        isPassed = BuiltIn().run_keyword_and_return_status("BuiltIn.Should Not Contain", actual, expected)
        if not isPassed:
            error_msg = "{expected_value} is not found in {actual_value}".format(expected_value=expected, actual_value=actual)
            Util.log_failure(self, self.sellib, error_msg)
    
    @keyword("Should Be Equal As Strings With Screenshot")
    def should_be_equal_as_strings_with_screenshot(self, actual, expected):
        """
        Validates `actual` equals `expected` value.
         - Captures screenshot if failed
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        isPassed = BuiltIn().run_keyword_and_return_status("BuiltIn.Should Be Equal As Strings", actual, expected)
        if not isPassed:
            error_msg = "{expected_value} is not same as {actual_value}".format(expected_value=expected, actual_value=actual)
            Util.log_failure(self, self.sellib, error_msg)

    @keyword("Reload Webpage")
    def reload_webpage(self):
        """
        Performs reload in webpage and waits until dom get loaded
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        self.sellib.reload_page()
        BuiltIn().run_keyword("SeleniumHelperLibrary.Wait Until DOM Loaded")


from .version import VERSION
from robot.libraries.BuiltIn import BuiltIn
from robot.api.deco import keyword
from robot.api import logger
from .util import Util

class TextareaHelper(Util):

    def __init__(self):
        pass
    
    @keyword("Textarea Field Value Should Be")
    def textarea_field_value_should_be(self, locator, text):
        """
        Wait for ``locator`` in webpage and validates given `text` present in textarea
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
            self.sellib.textarea_value_should_be(locator, text)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)

    @keyword("Textarea Field Value Should Contain")
    def textarea_field_value_should_contain(self, locator, text):
        """
        Wait for ``locator`` in webpage and validates given `text` contains in textarea
        """
        self.sellib = BuiltIn().get_library_instance('SeleniumLibrary')
        try:
            Util.wait_for_element(self, self.sellib, locator)
            self.sellib.textarea_should_contain(locator, text)
        except Exception as e:
            Util.log_failure(self, self.sellib, e)